#!/bin/bash

varshow()
{
(( prcount=${#varname[@]} ))
for ((i=0; i < prcount ; i++)) ;do
    echo "${varname[$i]}=${varvalue[$i]}"
done
echo
}

varget()
{
getindex=0
getflag=0
if [ -n "${varvalue[0]}" ]; then
    (( getsize=${#varvalue[@]} ))
else
    getsize=0
fi					
while [ $getindex -le $getsize ] && [ $getflag != 1 ]; do
    if [ "$*" = "${varname[$getindex]}" ]; then
    	echo "${varvalue[$getindex]}"
	getflag=1
    fi
    ((getindex++))
done
}

varset()
{
# usage - varset var=value - must use equals to set value
echo "$*" | grep = > /dev/null 2>&1
if [ "$?" -eq 0  ]; then
    setname=$(echo "$*" | awk -F "=" '{ print $1 }')
    setvalue=$(echo "$*" | awk -F "=" '{ print $2 }')
    if [ -n "$setname" ]; then
        setindex=0
	setflag=0
	if [ -n "${varvalue[0]}" ]; then
    	    (( setsize=${#varvalue[@]} ))
	else
	    setsize=0
	fi
        while [ $setindex -le $setsize ] && [ $setflag -ne 1 ]; do
	    if [ "${varname[$setindex]}" = "$setname" ]; then
		varvalue[$setindex]="$setvalue"
		setflag=1
	    fi
	    ((setindex++))
        done
	if [ $setflag -ne 1 ]; then
	    varname[$setsize]="$setname"
	    varvalue[$setsize]="$setvalue"
        fi
    fi
fi
}

varunset()
{
unindex=0
unflag=0
if [ -n "${varvalue[0]}" ]; then
    (( unsize=${#varvalue[@]} ))
else
    unsize=0
fi
(( lastvar=unsize - 1 ))
while [ $unindex -le $unsize ] && [ $unflag != 1 ]; do
    if [ "$*" = "${varname[$unindex]}" ]; then
        varname[$unindex]="${varname[$lastvar]}"
        varvalue[$unindex]="${varvalue[$lastvar]}"
        unset varname[$lastvar]
        unset varvalue[$lastvar]
	unflag=1
    fi
    ((unindex++))
done
}

readconf()
{
if ! [ -z $1 ]; then
    IFS=$'\n'
    conflines=$(${0%/*}/config.sed $1)
    for line in $conflines; do
	linename=$(echo "$line" | awk -F "=" '{ print $1 }')
	linevalue=$(echo "$line" | awk -F "=" '{ print $2 }')
	
	
	# check for duplicates in tmpname array
	# first declaration in a .conf file is the one used
	# later declarations are discarded
	s=0
	(( vcount=${#tmpvalue[@]} - 1 ))
	until [ $s -gt $vcount ] || [ "$linename" = "${tmpname[$s]}" ]; do
	    ((s++))
	done
	if ! [ "$linename" = "${tmpname[$s]}" ]; then
	    tmpname=(${tmpname[@]} $linename)
	    tmpvalue=(${tmpvalue[@]} $linevalue)
	fi
	
	# check for duplicates in varname array
	# most recenly read .conf file has priority
	# duplicate variables declared in previous .conf files are overwritten
	checkvalue="$(varget "$linename")"
	
	
	if ! [ -z "$checkvalue" ]; then
	    varunset "$linename"
	fi

    done
    lastline="${tmpname[${#tmpname[@]}-1]}"
	if [ "$lastline" -eq 0 ]; then
	    unset tmpname[${#tmpname[@]}-1]
	    varname=(${varname[@]} ${tmpname[@]})
	    varvalue=(${varvalue[@]} ${tmpvalue[@]})
	    unset tmpname; unset tmpvalue
	else
	    echo "config.sed: Error ${tmpname[${#tmpname[@]}-1]} in $1 on line ${tmpname[${#tmpname[@]}-2]}">&2
	    echo "Skipping config file $1">&2
	    unset tmpname; unset tmpvalue
	fi	
        unset IFS	
fi
}

toupper()
{
echo "$@" | sed -e y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/
}

tolower()
{
echo "$@" | sed -e y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/
}

filter()
{
# [a-z] on utf-8 linuxes includes more, e.g. äöü...
echo "$@" | sed -e "s/[^abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0-9._ ,:\/@-]//g"
}

namefilter()
{
echo $(tolower $(echo "$@" | sed -e "s/[^abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0-9.-]//g"))
}

nvfilter()
{
echo "$@" | sed -e "s/[^abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0-9._ ,:\/@%+= -]//g"
}

unescape()
{
  echo "$*"|awk '
  BEGIN {
    hextab["0"]=0; hextab["1"]=1;
    hextab["2"]=2; hextab["3"]=3;
    hextab["4"]=4; hextab["5"]=5;
    hextab["6"]=6; hextab["7"]=7;
    hextab["8"]=8; hextab["9"]=9;
    hextab["A"]=hextab["a"]=10;
    hextab["B"]=hextab["b"]=11;
    hextab["C"]=hextab["c"]=12;
    hextab["D"]=hextab["d"]=13;
    hextab["E"]=hextab["e"]=14;
    hextab["F"]=hextab["f"]=15;
  }
  {
    decoded="";
    i=1;
    len=length($0);
    while (i<=len) {
      c=substr($0, i, 1);
      if (c=="%") {
        if (i+2<=len) {
          c1=substr($0, i+1, 1);
          c2=substr($0, i+2, 1);
          if (hextab[c1]==""||hextab[c2]=="") {
            print "WARNING: invalid hex encoding: %" c1 c2|"cat>&2";
          }
          else {
            code=0+hextab[c1]*16+hextab[c2]+0;
            c=sprintf("%c", code);
            i=i+2;
          }
        }
        else {
          print "WARNING: invalid % encoding: " substr($0, i, len-i);
        }
      }
      else if (c=="+") {
        c=" ";
      }
      decoded=decoded c;
      ++i;
    }
    print decoded;
  }'
}
